/*
 * (c) Copyright IBM Corp. 2000, 2001, 2002.
 * All Rights Reserved.
 */
package org.eclipse.jdt.internal.core.eval;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.eval.model.ICompiledExpression;
import org.eclipse.jdt.core.eval.model.IRuntimeContext;
import org.eclipse.jdt.core.eval.model.IValue;

/**
 * @version 	1.0
 * @author
 */
public class InstructionSequence implements ICompiledExpression {

	private List fInstructions;
	
	public InstructionSequence() {
		fInstructions= new ArrayList(10);
	}
	
	/*
	 * @see ICompiledExpression#evaluate(IRuntimeContext)
	 */
	public IValue evaluate(IRuntimeContext context) throws CoreException {
		Interpreter interpreter= new Interpreter(getInstructions(), context);
		interpreter.execute();
		return interpreter.getResult();
	}

	/**
	 * Answers the array of instructions, or an empty array.
	 */
	private Instruction[] getInstructions() {
		int size= fInstructions.size();
		Instruction[] instructions= new Instruction[size];
		if (size > 0) {
			fInstructions.toArray(instructions);
		}
		return instructions;
	}
	
	/**
	 * Answer the instruction at the given address
	 */
	public Instruction getInstruction(int address) {
		return (Instruction)fInstructions.get(address);
	}
	
	/**
	 * Add the given instruction to the end of the list
	 */
	public void add(Instruction instruction) {
		fInstructions.add(instruction);
	}
	
	/**
	 * Answers true if there are no instructions in this sequence
	 */
	public boolean isEmpty() {
		return fInstructions.isEmpty();
	}

	/**
	 * Inserts the instruction at the given index.  If
	 * the index is less than 0 or greater than the current
	 * instruction count, the instruction is added at the end
	 * of the sequence.
	 * 
	 * Instructs the instructions to update their program counters.
	 */
	public void insert(Instruction instruction, int index) {
		fInstructions.add(index, instruction);
	}
	
	public Instruction get(int address) {
		return (Instruction)fInstructions.get(address);
	}
	
	public int getEnd() {
		return fInstructions.size() - 1;
	}
}
